import React, { useState, useEffect, useRef } from 'react';
import { Form, Input, Button, message, Descriptions, Spin, Tree, Tag, Select } from 'antd';
import { connect, history } from 'umi';
import { EditOutlined, LeftOutlined, CheckCircleOutlined, RightOutlined } from '@ant-design/icons';

import './Account.less';
import { zhCnFaci, enUsFaci } from '@/utils/power';
import { AccountTip } from '@/utils/tip';

import SelectCommunity from '@/components/SelectCommunity/Tow';
import { getNumber } from '@/utils/string'; // 正则
import { RA } from '@/utils/method';

const Account = (props: any) => {
  const module = 'Account';

  const { dispatch, SubResult, CommunityList, loading, user, DataSave, Permission, Info } = props;

  // 权限列表
  const treeData = localStorage.getItem('umi_locale') == 'zh-CN' ? zhCnFaci : enUsFaci;

  const [expandedKeys, setExpandedKeys] = useState<string[]>([]); // 展开栏目
  const [checkedKeys, setCheckedKeys] = useState([] as any); // 默认已选栏目
  const [selectedKeys, setSelectedKeys] = useState<string[]>([]); // 设置选中的树节点
  const [autoExpandParent, setAutoExpandParent] = useState<boolean>(true); // 树形菜单展开关闭
  const [treeDataList, settreeDataList] = useState(treeData); // 权限列表

  // 单选 二级还是三级管理员
  const [values, setvalues] = useState(2); // 树形菜单展开关闭
  const [towAccountId, settowAccountId] = useState(null); // 二级账户id

  //默认选中的小区
  const [community, setCommunity] = useState([] as any);

  // 表单标识
  const [form] = Form.useForm();
  const formRef = useRef(null);

  // 当前账户权限
  useEffect(() => {
    if (user != null) {
      let treeDatas = treeData;
      let _a = user.currentUser.permission; // 当前账户权限
      let _p = treeDatas[0].children; // 所有权限列表

      let arr: any = []; // 一级栏目权限
      let erArr: any = []; // 一级栏目权限
      // 循环一级 判断当前账户有没有权限修改增加
      for (let i of _a) {
        for (let j in _p) {
          if (_p[j].key == i) {
            arr.push(_p[j].key);
            _p[j].disabled = false;
          }
          // 二级权限
          for (let k in _p[j].children) {
            if (_p[j].children[k].key == i) {
              erArr.push(_p[j].children[k].key);
              _p[j].children[k].disableCheckbox = false;
            }
          }
        }
      }
      // 赋值改变
      treeDatas[0].children = _p;
      setCheckedKeys(treeDatas as any); // 可勾选列表
    }
  }, [user]);

  // 当前编辑账户的数据
  useEffect(() => {
    if (DataSave != null && user != null) {
      // 赋值
      form.setFieldsValue({
        tosUserName: DataSave.tosUserName,
        tosUserPhone: DataSave.tosUserPhone,
        tosAccountName: DataSave.tosAccountName,
      });

      setvalues(DataSave.tosUserLevel); // 等级
      setCommunity([...DataSave.tosUserServiceCell.split(',')]); // 默认小区

      // 当前登录的不是admin 就 给默认id
      if (user.currentUser.userModel.tosUserLevel > 1) {
        settowAccountId(DataSave.userLeader); // 二级账户id
      }

      if (DataSave.tosUserLevel == 3) {
        settowAccountId(DataSave.userLeader); //上级id
      }

      // 发起获取权限请求
      RA(54, { tosUserName: DataSave.tosUserName }, module, dispatch); // 发起获取权限请求
    } else {
      history.push('/AccountManagement/account');
    }
  }, [DataSave, user]);

  // 提交信息结果
  useEffect(() => {
    if (SubResult != null) {
      if (SubResult.statr == 0 && SubResult.desc == 0) {
        // 保存成功， 清掉原来数据 并且跳转
        message.success('Submitted Successfully !');
        dispatch({ type: module + '/ReData' });
        dispatch({ type: module + '/AccountClear' });
        history.go(-1);
      }
    }
  }, [SubResult]);

  // 已选权限
  useEffect(() => {
    if (Permission != null) {
      let newPer = eval('(' + Permission + ')');
      let data: any = []; // 编辑的账户的权限
      for (let i in newPer) {
        data[i] = newPer[i] + '';
      }

      let _p = treeData[0].children; // 所有权限列表
      // console.log(_p); // 所有权限列表

      for (let i in _p) {
        let arr = _p[i].children; // 每个权限
        let newArr = []; // 每个权限下的子权限列表
        for (let j in arr) {
          newArr.push(arr[j].key);
        }
        // console.log(newArr);

        if (!newArr.every((val) => data.includes(val))) {
          // 判断数组是否包含另一个数组
          // console.log('不包含' + _p[i].key);
          // 不全部包含就删除数组的元素
          var index = data.indexOf(_p[i].key);
          if (index > -1) {
            data.splice(index, 1);
          }
        } else {
          // console.log('包含' + _p[i].key);
        }
      }

      // 再赋值给列表
      // data.push('0');
      setCheckedKeys(data);
    }
  }, [Permission]);

  // 二级管辖小区和权限
  useEffect(() => {
    if (Info != null) {
      setCommunity(Info.community);
      power(Info.permissions);
    }
  }, [Info]);

  //权限赋值
  const power = (val: any) => {
    let _tmp = JSON.stringify(treeData); //将对象转换为json字符串形式
    let treeDatas = JSON.parse(_tmp); //将转换而来的字符串转换为原生js对

    let _a = val; // 当前账户权限
    let _p = treeDatas[0].children; // 所有权限列表

    let arr: any = []; // 一级栏目权限
    let erArr: any = []; // 一级栏目权限
    // 循环一级 判断当前账户有没有权限修改增加
    for (let i of _a) {
      for (let j in _p) {
        if (_p[j].key == i) {
          arr.push(_p[j].key);
          _p[j].disabled = false;
        }
        // 二级权限
        for (let k in _p[j].children) {
          if (_p[j].children[k].key == i) {
            erArr.push(_p[j].children[k].key);
            _p[j].children[k].disableCheckbox = false;
          }
        }
      }
    }
    // 且禁用最后两个账户管理
    _p[_p.length - 1].children[2].disableCheckbox = true;
    _p[_p.length - 1].children[3].disableCheckbox = true;
    // 赋值改变
    treeDatas[0].children = _p;
    settreeDataList(treeDatas as any); // 可勾选列表
  };

  // 保存提交
  const onFinishContract = async (value: any) => {
    const userInfo = localStorage.getItem('userInfo') || '';
    const id = JSON.parse(userInfo).userModel.id;
    if (
      value.tosAccountName.length <= 2 &&
      value.tosUserPhone.length != 8 &&
      value.tosUserPhone.length != 11
    ) {
      // 姓名联系方式
      message.error('Please enter the correct name and contact information!');
      return false;
    } else if (checkedKeys[0].key) {
      // 如果存在key  就是没有选择
      // 权限
      message.error('Please Select Permission!');
      return false;
    } else if (value.community.value == null || value.community.value.length == 0) {
      // 小区
      message.error('Please Select The Jurisdiction Area!');
      return false;
    } else if (value.tosUserName.length < 6) {
      // 账号长度
      message.error('The Account Password is Greater Than 6 Digits!');
      return false;
    } else if (values == 3 && towAccountId == null) {
      // 如果创建的是三级账号就要判断是否选择了绑定的二级账户
      message.error('Please select affiliated account !');
      return false;
    } else {
      value.tosUserServiceCellList = value.community.value; // 管辖小区
      value.tosUserEmail = value.tosUserName; //  邮箱就是账号
      value.tosUserLevel = values; //级别
      // value.creatorName = JSON.parse(userInfo).userModel.tosUserName; //新建者账号
      value.creatorId = id; //新建者ID
      if (values == 3) {
        value.userLeader = towAccountId;
      }
      delete value.community;
      DataSave ? (value.id = DataSave.id) : '';

      // 判断有没有 0  有就删除
      for (var i in checkedKeys) {
        checkedKeys[i] == '0' ? checkedKeys.splice(i, 1) : '';
      }

      // 追加到上级id
      let Obj = treeData[0].children;
      console.log(Obj);
      for (let i in Obj) {
        for (let j in checkedKeys) {
          if (JSON.stringify(Obj[i].children).indexOf(JSON.stringify(checkedKeys[j])) != -1) {
            // 提取出上级id  并且追加
            checkedKeys.push(Obj[i].key);
          }
        }
      }
      // 去重后在排序
      let newData = Array.from(new Set(checkedKeys)).sort((n1: any, n2: any) => {
        return parseInt(n1) - parseInt(n2);
      });

      // 另传权限
      let obj = {
        userName: value.tosUserName,
        userPassword: value.tosUserPwd,
        permissionArray: newData,
      };

      RA(38, value, module, dispatch); // 信息上传
      RA(42, obj, module, dispatch); // 权限上传
      return true;
    }
  };

  //goToReturn
  const goToReturn = () => {
    history.go(-1);
  };

  // 展开/收起树形菜单触发
  const onExpand = (expandedKeys: any) => {
    setExpandedKeys(expandedKeys);
    setAutoExpandParent(false);
  };

  // 点击单个触发
  const onCheck = (checkedKeys: any) => {
    // console.log('onCheck', checkedKeys);
    setCheckedKeys(checkedKeys);
  };

  // 点击树节点触发  - 大栏目
  const onSelect = (selectedKeys: any, info: any) => {
    // console.log('onSelect', info);
    setSelectedKeys(selectedKeys);
  };

  //手机号
  const keyup_communityManagerFee = (e: any) => {
    e.target.value = keyup_tool(e.target.value);
  };
  const keyup_tool = (value: any) => {
    return getNumber(value.replace(/[^\d^\.]+/g, ''));
  };

  return (
    <Spin spinning={loading}>
      <div className="contop" style={{ padding: '12px 20px' }}>
        <h3 className="capi">
          <EditOutlined />
          &nbsp; Edit Account
          <div className="back">
            <Button onClick={goToReturn}>
              <LeftOutlined />
              Back
            </Button>
          </div>
        </h3>
        <hr></hr>
        <Form
          ref={formRef}
          form={form}
          autoComplete="off"
          // layout="inline"
          layout="horizontal"
          name="contract"
          onFinish={onFinishContract}
        >
          <Descriptions column={{ xs: 1, sm: 2, md: 3 }}>
            <Descriptions.Item>
              <Form.Item name="tosUserName" label="Account ID" rules={AccountTip[0] as any}>
                <Input placeholder="Login Account" className="input" />
              </Form.Item>
            </Descriptions.Item>
          </Descriptions>

          <Descriptions column={{ xs: 1, sm: 2, md: 3 }}>
            <Descriptions.Item>
              <Form.Item name="tosAccountName" label="Name" rules={AccountTip[1] as any}>
                <Input placeholder="Name Of Administrator" className="input" />
              </Form.Item>
            </Descriptions.Item>
            <Descriptions.Item>
              <Form.Item name="tosUserPhone" label="Phone" rules={AccountTip[2] as any}>
                <Input
                  placeholder="Contact Information"
                  className="input"
                  maxLength={11}
                  onKeyUp={keyup_communityManagerFee}
                />
              </Form.Item>
            </Descriptions.Item>
          </Descriptions>

          <Form.Item name="community" label="Community">
            <SelectCommunity
              checklist={community}
              list={
                DataSave && DataSave.userLeaderCommunityNames
                  ? DataSave.userLeaderCommunityNames
                  : CommunityList
              }
            />
          </Form.Item>

          <div className="diy" style={{ marginBottom: '14px' }}>
            <div className="label">
              <span className="title">Privilege Level：</span>
            </div>
            <div className="label">
              <Tag
                icon={<CheckCircleOutlined />}
                color="success"
                style={{ verticalAlign: 'sub', fontSize: 14, padding: '4px 7px' }}
              >
                {DataSave && DataSave.tosUserLevel == 2
                  ? 'Two Level Administrator '
                  : 'Three Level Administrator'}
              </Tag>
              {DataSave && DataSave.tosUserLevel == 3 ? (
                <Tag
                  icon={<RightOutlined />}
                  color="processing"
                  style={{ verticalAlign: 'sub', fontSize: 14, padding: '4px 7px' }}
                >
                  Superiors：{DataSave.userLeaderAccountName} - {DataSave.userLeaderName}
                </Tag>
              ) : (
                ''
              )}
            </div>
          </div>

          <div className="diy" style={{ marginBottom: '24px' }}>
            <div className="label">
              <span className="title">Permission List：</span>
            </div>
            <div className="label">
              <Tree
                checkable
                onExpand={onExpand}
                expandedKeys={expandedKeys}
                autoExpandParent={autoExpandParent}
                onCheck={onCheck}
                checkedKeys={checkedKeys}
                onSelect={onSelect}
                selectedKeys={selectedKeys}
                treeData={treeDataList}
                disabled
              />
            </div>
          </div>

          <div className="diy">
            <div className="label"></div>
            <div className="label">
              <Button type="primary" htmlType="submit" loading={loading}>
                Submit
              </Button>
            </div>
          </div>
        </Form>
      </div>
    </Spin>
  );
};

const AccountProps = (state: any) => {
  const { Data, DataSave, DataSaveDetail, Result, SubResult, Permission, Info } = state.Account;
  const { CommunityList } = state.Init; // 小区列表
  const loading = state.loading.models.Account || false;
  const { user } = state;
  return {
    Data,
    DataSave,
    DataSaveDetail,
    Result,
    loading,
    Permission,
    CommunityList,
    user, // 获取当前账户信息
    SubResult,
    Info,
  };
};

export default connect(AccountProps)(Account);
